@@ -88,8 +88,7 @@ var gPrivacyPane = {
   initializeHistoryMode: function()
   {
     let mode;
-    let getVal = function(aPref)
-      document.getElementById(aPref).value;
+    let getVal = aPref => document.getElementById(aPref).value;
 
     if (this._checkDefaultValues(this.prefsForDefault)) {
       if (getVal("browser.privatebrowsing.autostart"))
