@@ -44,7 +44,9 @@
 #endif
 
 #ifdef MOZ_WIDGET_COCOA
+#ifdef MOZ_ENABLE_NPAPI
 #include "PluginInterposeOSX.h"
+#endif // MOZ_ENABLE_NPAPI
 #include "PluginUtilsOSX.h"
 #endif
 
@@ -133,7 +135,7 @@ PluginModuleChild::PluginModuleChild(bool aIsChrome)
         gChromeInstance = this;
     }
 
-#ifdef XP_MACOSX
+#if defined(XP_MACOSX) && defined(MOZ_ENABLE_NPAPI)
     if (aIsChrome) {
       mac_plugin_interposing::child::SetUpCocoaInterposing();
     }
@@ -2548,6 +2550,7 @@ PluginModuleChild::ProcessNativeEvents() {
 }
 #endif
 
+#ifdef MOZ_ENABLE_NPAPI
 NPError
 PluginModuleChild::PluginRequiresAudioDeviceChanges(
                           PluginInstanceChild* aInstance,
@@ -2606,3 +2609,4 @@ PluginModuleChild::RecvNPP_SetValue_NPNVaudioDeviceChangeDetails(
     return false;
 #endif
 }
+#endif // MOZ_ENABLE_NPAPI
