@@ -90,7 +90,6 @@ namespace jit {
     _(Random)                                   \
     _(StringSplit)                              \
     _(NaNToZero)                                \
-    _(RegExpMatcher)                            \
     _(RegExpSearcher)                           \
     _(RegExpTester)                             \
     _(StringReplace)                            \
@@ -494,14 +493,6 @@ class RNaNToZero final : public RInstruction
     bool recover(JSContext* cx, SnapshotIterator& iter) const override;
 };
 
-class RRegExpMatcher final : public RInstruction
-{
-  public:
-    RINSTRUCTION_HEADER_NUM_OP_(RegExpMatcher, 3)
-
-    [[nodiscard]] bool recover(JSContext* cx, SnapshotIterator& iter) const override;
-};
-
 class RRegExpSearcher final : public RInstruction
 {
   public:
