@@ -1105,29 +1105,6 @@ MacroAssembler::branchTestMagicImpl(Condition cond, const T& t, L label)
     j(cond, label);
 }
 
-// ========================================================================
-// Canonicalization primitives.
-void
-MacroAssembler::canonicalizeFloat32x4(FloatRegister reg, FloatRegister scratch)
-{
-    ScratchSimd128Scope scratch2(*this);
-
-    MOZ_ASSERT(scratch.asSimd128() != scratch2.asSimd128());
-    MOZ_ASSERT(reg.asSimd128() != scratch2.asSimd128());
-    MOZ_ASSERT(reg.asSimd128() != scratch.asSimd128());
-
-    FloatRegister mask = scratch;
-    vcmpordps(Operand(reg), reg, mask);
-
-    FloatRegister ifFalse = scratch2;
-    float nanf = float(JS::GenericNaN());
-    loadConstantSimd128Float(SimdConstant::SplatX4(nanf), ifFalse);
-
-    bitwiseAndSimd128(Operand(mask), reg);
-    bitwiseAndNotSimd128(Operand(ifFalse), mask);
-    bitwiseOrSimd128(Operand(mask), reg);
-}
-
 // ========================================================================
 // Memory access primitives.
 void
