@@ -4,16 +4,17 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch07/7.3/7.3-12.js
- * @description 7.3 - ES5 specifies that a multiline comment that contains a line terminator character <PS> (\u2029) must be treated as a single line terminator for the purposes of semicolon insertion
+ * @path ch07/7.6/7.6-32.js
+ * @description 7.6 - SyntaxError expected: reserved words used as Identifier Names in UTF8: enum (enum)
  */
 
 
 function testcase() {
-        /*MultiLine
-        Comments 
-        \u2029 var = ;
-        */
-        return true;
+            try {
+                eval("var \u0065\u006e\u0075\u006d = 123;");  
+                return false;
+            } catch (e) {
+                return e instanceof SyntaxError;  
+            }
     }
 runTestCase(testcase);
