@@ -4,21 +4,19 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch07/7.8/7.8.4/7.8.4-5-s.js
- * @description An OctalEscapeSequence is not allowed in a String under Strict Mode
+ * @path ch07/7.8/7.8.3/7.8.3-7-s.js
+ * @description Strict Mode - octal extension (005) is forbidden in strict mode
  * @onlyStrict
  */
 
 
-function testcase()
-{
-  try 
-  {
-    eval('"use strict"; var x = "\\00a";');
-    return false;
-  }
-  catch (e) {
-    return (e instanceof SyntaxError);
-  }
- }
+function testcase() {
+        "use strict";
+        try {
+            eval("var _7_8_3_7 = 005;");
+            return false;
+        } catch (e) {
+            return e instanceof SyntaxError && typeof _7_8_3_7 === "undefined";
+        }
+    }
 runTestCase(testcase);
