@@ -4,15 +4,16 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch08/8.12/8.12.1/8.12.1-1_34.js
- * @description Properties - [[HasOwnProperty]] (non-configurable, non-enumerable own getter/setter property)
+ * @path ch08/8.12/8.12.1/8.12.1-1_45.js
+ * @description Properties - [[HasOwnProperty]] (configurable, enumerable inherited setter property)
  */
 
 function testcase() {
 
-    var o = {};
-    Object.defineProperty(o, "foo", {get: function() {return 42;}, set: function() {;}});
-    return o.hasOwnProperty("foo");
+    var base = {};
+    Object.defineProperty(base, "foo", {set: function() {;}, enumerable:true, configurable:true});
+    var o = Object.create(base);
+    return o.hasOwnProperty("foo")===false;
 
 }
 runTestCase(testcase);
