@@ -4,27 +4,23 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch12/12.10/12.10-0-11.js
- * @description with introduces scope - name lookup finds inner variable
+ * @path ch12/12.10/12.10-0-7.js
+ * @description with introduces scope - scope removed when exiting with statement
  */
 
 
 function testcase() {
-  function f(o) {
+  var o = {foo: 1};
 
-    function innerf(o) {
-      var x = 42;
-
-      with (o) {
-        return x;
-      }
-    }
-
-    return innerf(o);
+  with (o) {
+    foo = 42;
   }
-  
-  if (f({}) === 42) {
-    return true;
+
+  try {
+    foo;
+  }
+  catch (e) {
+     return true;
   }
  }
 runTestCase(testcase);
