@@ -5,8 +5,8 @@
  * No matter how control leaves the embedded 'Statement',
  * the scope chain is always restored to its former state
  *
- * @path ch12/12.10/S12.10_A3.4_T4.js
- * @description Using "with" statement within iteration statement, leading to completion by break
+ * @path ch12/12.10/S12.10_A3.8_T4.js
+ * @description Declaring function constructor within "with" statement, leading to completion by exception
  * @noStrict
  */
 
@@ -20,28 +20,39 @@ var myObj = {
     valueOf : function(){return 'obj_valueOf';}
 }
 
-do {
+try {
     with(myObj){
-        p1 = 'x1';
-        break;
+        var __FACTORY = function(){
+            p1 = 'x1';
+            throw value;
+        }
+        var obj = new __FACTORY;
     }
-} while(false);
+} catch(e){
+    result = p1;
+}
 
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#1
-if(p1 !== 1){
-  $ERROR('#1: p1 === 1. Actual:  p1 ==='+ p1  );
+if(result !== 1){
+  $ERROR('#1: result === 1. Actual:  result ==='+ result  );
 }
 //
 //////////////////////////////////////////////////////////////////////////////
 
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#2
-if(myObj.p1 !== "x1"){
-  $ERROR('#2: myObj.p1 === "x1". Actual:  myObj.p1 ==='+ myObj.p1  );
+if(p1 !== 1){
+  $ERROR('#2: p1 === 1. Actual:  p1 ==='+ p1  );
 }
 //
 //////////////////////////////////////////////////////////////////////////////
 
-
+//////////////////////////////////////////////////////////////////////////////
+//CHECK#3
+if(myObj.p1 !== "x1"){
+  $ERROR('#3: myObj.p1 === "x1". Actual:  myObj.p1 ==='+ myObj.p1  );
+}
+//
+//////////////////////////////////////////////////////////////////////////////
 
