@@ -4,21 +4,19 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch12/12.2/12.2.1/12.2.1-8-s.js
- * @description eval - a direct eval assigning into 'eval' throws SyntaxError in strict mode
+ * @path ch12/12.2/12.2.1/12.2.1-17-s.js
+ * @description A Function constructor (called as a function) assigning into 'arguments' will not throw any error if contained within strict mode and its body does not start with strict mode
  * @onlyStrict
  */
 
 
+
+
 function testcase() {
   'use strict';
-
-  try {
-    eval('eval = 42;');
-    return false;
-  }
-  catch (e) {
-    return (e instanceof SyntaxError) ;
-  }
- }
-runTestCase(testcase);
+  
+    var f = Function('arguments = 42;');
+    f();
+    return true;
+}
+runTestCase(testcase);
\ No newline at end of file
