@@ -1,6 +1,7 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  *
  * Copyright 2016 Mozilla Foundation
+ * Copyright 2023 Moonchild Productions
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -27,14 +28,14 @@ class StringBuffer;
 namespace wasm {
 
 template<size_t base>
-MOZ_MUST_USE bool
+[[nodiscard]] bool
 RenderInBase(StringBuffer& sb, uint64_t num);
 
 template<class T>
 class Raw;
 
 template<class T>
-MOZ_MUST_USE bool
+[[nodiscard]] bool
 RenderNaN(StringBuffer& sb, Raw<T> num);
 
 }  // namespace wasm
