@@ -102,6 +102,7 @@ static const VideoSizeAbbr video_size_abbrs[] = {
     { "wsxga",    1600,1024 },
     { "wuxga",    1920,1200 },
     { "woxga",    2560,1600 },
+    { "wqhd",     2560,1440 },
     { "wqsxga",   3200,2048 },
     { "wquxga",   3840,2400 },
     { "whsxga",   6400,4096 },
@@ -111,6 +112,7 @@ static const VideoSizeAbbr video_size_abbrs[] = {
     { "hd480",     852, 480 },
     { "hd720",    1280, 720 },
     { "hd1080",   1920,1080 },
+    { "quadhd",   2560,1440 },
     { "2k",       2048,1080 }, /* Digital Cinema System Specification */
     { "2kdci",    2048,1080 },
     { "2kflat",   1998,1080 },
