@@ -3,27 +3,28 @@
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
-#if JPEGXL_ENABLE_APNG
-
 #include "tools/benchmark/benchmark_codec_png.h"
 
 #include <stddef.h>
 #include <stdint.h>
 
 #include <string>
+#include <vector>
 
 #include "lib/extras/codec.h"
 #include "lib/extras/dec/apng.h"
+#include "lib/extras/dec/decode.h"
 #include "lib/extras/enc/apng.h"
-#include "lib/extras/packed_image.h"
-#include "lib/extras/packed_image_convert.h"
 #include "lib/extras/time.h"
-#include "lib/jxl/base/padded_bytes.h"
 #include "lib/jxl/base/span.h"
-#include "lib/jxl/base/thread_pool_internal.h"
-#include "lib/jxl/codec_in_out.h"
+#include "lib/jxl/base/status.h"
+#include "tools/benchmark/benchmark_args.h"
+#include "tools/benchmark/benchmark_codec.h"
+#include "tools/speed_stats.h"
+#include "tools/thread_pool_internal.h"
 
-namespace jxl {
+namespace jpegxl {
+namespace tools {
 
 struct PNGArgs {
   // Empty, no PNG-specific args currently.
@@ -40,37 +41,38 @@ class PNGCodec : public ImageCodec {
 
   Status ParseParam(const std::string& param) override { return true; }
 
-  Status Compress(const std::string& filename, const CodecInOut* io,
-                  ThreadPoolInternal* pool, std::vector<uint8_t>* compressed,
+  Status Compress(const std::string& filename, const PackedPixelFile& ppf,
+                  ThreadPool* pool, std::vector<uint8_t>* compressed,
                   jpegxl::tools::SpeedStats* speed_stats) override {
-    const size_t bits = io->metadata.m.bit_depth.bits_per_sample;
-    const double start = Now();
-    JXL_RETURN_IF_ERROR(Encode(*io, extras::Codec::kPNG, io->Main().c_current(),
-                               bits, compressed, pool));
-    const double end = Now();
+    const double start = jxl::Now();
+    JXL_RETURN_IF_ERROR(
+        jxl::Encode(ppf, jxl::extras::Codec::kPNG, compressed, pool));
+    const double end = jxl::Now();
     speed_stats->NotifyElapsed(end - start);
     return true;
   }
 
   Status Decompress(const std::string& /*filename*/,
-                    const Span<const uint8_t> compressed,
-                    ThreadPoolInternal* pool, CodecInOut* io,
+                    const Span<const uint8_t> compressed, ThreadPool* pool,
+                    PackedPixelFile* ppf,
                     jpegxl::tools::SpeedStats* speed_stats) override {
-    extras::PackedPixelFile ppf;
-    const double start = Now();
-    JXL_RETURN_IF_ERROR(extras::DecodeImageAPNG(
-        compressed, extras::ColorHints(), SizeConstraints(), &ppf));
-    const double end = Now();
+    const double start = jxl::Now();
+    JXL_RETURN_IF_ERROR(jxl::extras::DecodeImageAPNG(
+        compressed, jxl::extras::ColorHints(), ppf));
+    const double end = jxl::Now();
     speed_stats->NotifyElapsed(end - start);
-    JXL_RETURN_IF_ERROR(ConvertPackedPixelFileToCodecInOut(ppf, pool, io));
     return true;
   }
 };
 
 ImageCodec* CreateNewPNGCodec(const BenchmarkArgs& args) {
-  return new PNGCodec(args);
+  if (jxl::extras::GetAPNGEncoder() &&
+      jxl::extras::CanDecode(jxl::extras::Codec::kPNG)) {
+    return new PNGCodec(args);
+  } else {
+    return nullptr;
+  }
 }
 
-}  // namespace jxl
-
-#endif
+}  // namespace tools
+}  // namespace jpegxl
