@@ -327,7 +327,7 @@ public:
   DeleterType& get_deleter() { return del(); }
   const DeleterType& get_deleter() const { return del(); }
 
-  MOZ_MUST_USE Pointer release()
+  [[nodiscard]] Pointer release()
   {
     Pointer p = ptr();
     ptr() = nullptr;
@@ -462,7 +462,7 @@ public:
   DeleterType& get_deleter() { return mTuple.second(); }
   const DeleterType& get_deleter() const { return mTuple.second(); }
 
-  MOZ_MUST_USE Pointer release()
+  [[nodiscard]] Pointer release()
   {
     Pointer p = mTuple.first();
     mTuple.first() = nullptr;
