@@ -59,14 +59,8 @@ extern LazyLogModule gPIPNSSLog;
 static nsresult
 attemptToLogInWithDefaultPassword()
 {
-#ifdef NSS_DISABLE_DBM
   // The SQL NSS DB requires the user to be authenticated to set certificate
-  // trust settings, even if the user's password is empty. To maintain
-  // compatibility with the DBM-based database, try to log in with the
-  // default empty password. This will allow, at least, tests that need to
-  // change certificate trust to pass on all platforms. TODO(bug 978120): Do
-  // proper testing and/or implement a better solution so that we are confident
-  // that this does the correct thing outside of xpcshell tests too.
+  // trust settings, even if the user's password is empty.
   UniquePK11SlotInfo slot(PK11_GetInternalKeySlot());
   if (!slot) {
     return MapSECStatus(SECFailure);
@@ -76,7 +70,6 @@ attemptToLogInWithDefaultPassword()
     // has a non-default password.
     Unused << PK11_InitPin(slot.get(), nullptr, nullptr);
   }
-#endif
 
   return NS_OK;
 }
