@@ -37,7 +37,11 @@ extern PRLogModuleInfo* sCocoaLog;
 
 extern void EnsureLogInitialized();
 
+#if defined(MAC_OS_X_VERSION_10_6) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6)
 extern NSPasteboard* globalDragPboard;
+#else
+extern NSPasteboardWrapper* globalDragPboard;
+#endif
 extern NSView* gLastDragView;
 extern NSEvent* gLastDragMouseDownEvent;
 extern bool gUserCancelledDrag;
@@ -52,6 +56,51 @@ NSString* const kCorePboardType_urld = @"CorePasteboardFlavorType 0x75726C64"; /
 NSString* const kCorePboardType_urln = @"CorePasteboardFlavorType 0x75726C6E"; // 'urln'  title
 NSString* const kUTTypeURLName = @"public.url-name";
 NSString* const kCustomTypesPboardType = @"org.mozilla.custom-clipdata";
+#if !defined(MAC_OS_X_VERSION_10_6) || (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_6)
+// bug 966986 attachment 8540201
+NSString* const kCorePboardType_text = @"CorePasteboardFlavorType 0x54455854"; // 'TEXT'  text
+
+// restore bug 966986
+@implementation NSPasteboardWrapper
+- (id)initWithPasteboard:(NSPasteboard*)aPasteboard
+{
+  if ((self = [super init])) {
+    mPasteboard = [aPasteboard retain];
+    mFilenames = nil;
+  }
+  return self;
+}
+
+- (id)propertyListForType:(NSString *)aType
+{
+  if (![aType isEqualToString:NSFilenamesPboardType]) {
+    return [mPasteboard propertyListForType:aType];
+  }
+
+  if (!mFilenames) {
+    mFilenames = [[mPasteboard propertyListForType:aType] retain];
+  }
+
+  return mFilenames;
+}
+
+- (NSPasteboard*) pasteboard
+{
+  return mPasteboard;
+}
+
+- (void)dealloc
+{
+  [mPasteboard release];
+  mPasteboard = nil;
+
+  [mFilenames release];
+  mFilenames = nil;
+
+  [super dealloc];
+}
+@end
+#endif
 
 nsDragService::nsDragService()
 {
@@ -219,6 +268,7 @@ nsDragService::ConstructDragImage(nsIDOMNode* aDOMNode,
   NS_OBJC_END_TRY_ABORT_BLOCK_NIL;
 }
 
+#if defined(MAC_OS_X_VERSION_10_6) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6)
 bool
 nsDragService::IsValidType(NSString* availableType, bool allowFileURL)
 {
@@ -290,6 +340,7 @@ nsDragService::GetFilePath(NSPasteboardItem* item)
 
   NS_OBJC_END_TRY_ABORT_BLOCK_NIL;
 }
+#endif
 
 // We can only invoke NSView's 'dragImage:at:offset:event:pasteboard:source:slideBack:' from
 // within NSView's 'mouseDown:' or 'mouseDragged:'. Luckily 'mouseDragged' is always on the
@@ -416,6 +467,8 @@ nsDragService::GetData(nsITransferable* aTransferable, uint32_t aItemIndex)
 
     MOZ_LOG(sCocoaLog, LogLevel::Info, ("nsDragService::GetData: looking for clipboard data of type %s\n", flavorStr.get()));
 
+    // revert bug 966986
+#if defined(MAC_OS_X_VERSION_10_6) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6)
     NSArray* droppedItems = [globalDragPboard pasteboardItems];
     if (!droppedItems) {
       continue;
@@ -430,9 +483,18 @@ nsDragService::GetData(nsITransferable* aTransferable, uint32_t aItemIndex)
     if (!item) {
       continue;
     }
+#endif
 
     if (flavorStr.EqualsLiteral(kFileMime)) {
+#if defined(MAC_OS_X_VERSION_10_6) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6)
       NSString* filePath = GetFilePath(item);
+#else
+      NSArray* pFiles = [globalDragPboard propertyListForType:NSFilenamesPboardType];
+      if (!pFiles || [pFiles count] < (aItemIndex + 1))
+        continue;
+
+      NSString* filePath = [pFiles objectAtIndex:aItemIndex];
+#endif
       if (!filePath)
         continue;
 
@@ -454,6 +516,7 @@ nsDragService::GetData(nsITransferable* aTransferable, uint32_t aItemIndex)
       
       break;
     }
+#if defined(MAC_OS_X_VERSION_10_6) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6)
     else if (flavorStr.EqualsLiteral(kCustomTypesMime)) {
       NSString* availableType = [item availableTypeFromArray:[NSArray arrayWithObject:kCustomTypesPboardType]];
       if (!availableType || !IsValidType(availableType, false)) {
@@ -479,7 +542,10 @@ nsDragService::GetData(nsITransferable* aTransferable, uint32_t aItemIndex)
       free(clipboardDataPtr);
       break;
     }
+#endif
 
+    // backout 966986 and use updated fix from attachment 8540201
+#if defined(MAC_OS_X_VERSION_10_6) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6)
     NSString* pString = nil;
     if (flavorStr.EqualsLiteral(kUnicodeMime)) {
       pString = GetStringForType(item, (const NSString*)kUTTypeUTF8PlainText);
@@ -502,6 +568,52 @@ nsDragService::GetData(nsITransferable* aTransferable, uint32_t aItemIndex)
       pString = GetStringForType(item, (const NSString*)kUTTypeRTF);
     }
     if (pString) {
+#else
+     NSString *pboardType = NSStringPboardType;
+
+     if (nsClipboard::IsStringType(flavorStr, &pboardType) ||
+         flavorStr.EqualsLiteral(kRTFMime) ||
+         flavorStr.EqualsLiteral(kURLMime) ||
+         flavorStr.EqualsLiteral(kURLDataMime) ||
+         flavorStr.EqualsLiteral(kURLDescriptionMime)) {
+      // The return value of [globalDragPboard stringForType:pboardType]
+      // contains the data of all items, so return the value only if
+      // aItemIndex is 0, and return empty string for others, to avoid
+      // returning duplicated data.
+      NSString* pString;
+      if (aItemIndex == 0) {
+        pString = [[globalDragPboard pasteboard] stringForType:pboardType];
+        if (!pString) {
+          // null is returned in some case, see bug 966986 comment 1.
+          // Try with another type.
+          if (flavorStr.EqualsLiteral(kURLMime))
+            pboardType = kCorePboardType_url;
+          else
+            pboardType = kCorePboardType_text;
+          pString = [[globalDragPboard pasteboard] stringForType:pboardType];
+          if (!pString)
+            continue;
+        }
+
+        if (flavorStr.EqualsLiteral(kURLMime)) {
+          // text/x-moz-url requires the title for each URL,
+          // We don't know which URL corresponds to which file, so
+          // duplicate all lines to make the URL itself as its title.
+          NSArray* lines = [pString componentsSeparatedByString:@"\n"];
+          NSMutableArray* namedLines = [[NSMutableArray alloc] init];
+          for (uint32_t j = 0, lineCount = [lines count]; j < lineCount; j ++) {
+            [namedLines addObject:[lines objectAtIndex:j]];
+            [namedLines addObject:[lines objectAtIndex:j]];
+          }
+          pString = [namedLines componentsJoinedByString:@"\n"];
+          [namedLines release];
+        }
+      }
+      else {
+        pString = @"";
+      }
+#endif
+
       NSData* stringData;
       if (flavorStr.EqualsLiteral(kRTFMime)) {
         stringData = [pString dataUsingEncoding:NSASCIIStringEncoding];
@@ -592,6 +704,8 @@ nsDragService::IsDataFlavorSupported(const char *aDataFlavor, bool *_retval)
     }
   }
 
+  // backout bug 966986
+#if defined(MAC_OS_X_VERSION_10_6) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6)
   const NSString* type = nil;
   bool allowFileURL = false;
   if (dataFlavor.EqualsLiteral(kFileMime)) {
@@ -616,6 +730,25 @@ nsDragService::IsDataFlavorSupported(const char *aDataFlavor, bool *_retval)
   if (availableType && IsValidType(availableType, allowFileURL)) {
     *_retval = true;
   }
+#else
+  NSString *pboardType = nil;
+
+  if (dataFlavor.EqualsLiteral(kFileMime)) {
+    NSString* availableType = [[globalDragPboard pasteboard] availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]];
+    if (availableType && [availableType isEqualToString:NSFilenamesPboardType])
+      *_retval = true;
+  }
+  else if (dataFlavor.EqualsLiteral(kURLMime)) {
+    NSString* availableType = [[globalDragPboard pasteboard] availableTypeFromArray:[NSArray arrayWithObject:kCorePboardType_url]];
+    if (availableType && [availableType isEqualToString:kCorePboardType_url])
+      *_retval = true;
+  }
+  else if (nsClipboard::IsStringType(dataFlavor, &pboardType)) {
+    NSString* availableType = [[globalDragPboard pasteboard] availableTypeFromArray:[NSArray arrayWithObject:pboardType]];
+    if (availableType && [availableType isEqualToString:pboardType])
+      *_retval = true;
+  }
+#endif
 
   return NS_OK;
 
@@ -635,10 +768,25 @@ nsDragService::GetNumDropItems(uint32_t* aNumItems)
     return NS_OK;
   }
 
+  // backout bug 966986
+#if defined(MAC_OS_X_VERSION_10_6) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6)
   NSArray* droppedItems = [globalDragPboard pasteboardItems];
   if (droppedItems) {
     *aNumItems = [droppedItems count];
   }
+#else
+  // if there is a clipboard and there is something on it, then there is at least 1 item
+  NSArray* clipboardTypes = [[globalDragPboard pasteboard] types];
+  if (globalDragPboard && [clipboardTypes count] > 0)
+    *aNumItems = 1;
+  else
+    return NS_OK;
+
+  // if there is a list of files, send the number of files in that list
+  NSArray* fileNames = [globalDragPboard propertyListForType:NSFilenamesPboardType];
+  if (fileNames)
+    *aNumItems = [fileNames count];
+#endif
 
   return NS_OK;
 
