@@ -226,7 +226,7 @@ nsPropertiesParser::ParseValueCharacter(char16_t aChar, const char16_t* aCur,
             aTokenStart = aCur + 1;
             break;
           }
-          MOZ_FALLTHROUGH;
+          [[fallthrough]];
 
         case '\r':
           // we're done! We have a key and value
