@@ -355,7 +355,7 @@ CloneArray(uint16_t aInType, const nsIID* aInIID,
       if (aOutIID) {
         *aOutIID = *aInIID;
       }
-      MOZ_FALLTHROUGH;
+      [[fallthrough]];
 
     case nsIDataType::VTYPE_INTERFACE: {
       memcpy(*aOutValue, aInValue, allocSize);
@@ -759,7 +759,7 @@ nsDiscriminatedUnion::ToString(nsACString& aOutString) const
     case nsIDataType::VTYPE_WSTRING_SIZE_IS:
     case nsIDataType::VTYPE_WCHAR:
       NS_ERROR("ToString being called for a string type - screwy logic!");
-      MOZ_FALLTHROUGH;
+      [[fallthrough]];
 
     // XXX We might want stringified versions of these... ???
 
