@@ -1232,7 +1232,8 @@ nsThread::SetObserver(nsIThreadObserver* aObs)
   }
 
   MutexAutoLock lock(mLock);
-  mObserver = aObs;
+  nsCOMPtr<nsIThreadObserver> observer = aObs;
+  mObserver.swap(observer);
   return NS_OK;
 }
 
